<?php

    if ( !defined( 'ABSPATH' ) ) exit;

    class JShiftSchedule698 extends stdClass
    {
        private $options = null;
        private $month = null;
        private $day_in_month = null;
        private $work_day_in_month = 0;
        private $check_works = true;
        private $silent = false;
        private $_start = 1;
        private $_end = 31;
        private $display_mode = 'month';
        private $category = 0;
        
        public function __construct($options, $silent = false)
        {
            $this->options = $options;
            $this->options['token'] = $this->getToken();

            if (!isset($this->options['month']) or ($this->options['month'] == '')) {
                if (isset($_REQUEST['month'])) {
                    $this->month = sanitize_text_field($_REQUEST['month']);
                } else {
                    if (isset($_COOKIE['shift_schedule'])) {
                        $this->month = sanitize_text_field($_COOKIE['shift_schedule']);
                    } else {
                        $this->month = date_i18n('Y-m');
                    }
                }
            } else {
                $this->month = $this->options['month'];
            }

            if (isset($this->options['week'])) {
                $this->display_mode = 'week';
                $this->options['info'] = 'hide';          
                if ($this->options['week'] != 'next') {
                    $now = time();
                } else {
                    $now = strtotime('+1 week');;
                }
                $day = date_i18n('N', $now);
                $this->_start = date_i18n('Y-m-d', strtotime('-'.($day-1).' day', $now));
                $this->_end = date_i18n('Y-m-d', strtotime('+'.(7-$day).' day', $now));
                $this->month = date_i18n('Y-m', $now);

            } else {
                $this->_start = date_i18n('Y-m-d', strtotime($this->month.'-01'));
                $this->_end = date_i18n('Y-m-t', strtotime($this->month.'-01'));
            }
            
            if (isset($_REQUEST['shift-schedule-category-selector'])) {
                $this->category = sanitize_text_field($_REQUEST['shift-schedule-category-selector']);
            }

            $this->silent = $silent;
            
        }

        private function getToken()
        {
            $chars = 'ABCDEFGHIJKLMNOPQURSTXYVWZ0123456789';
            $token = '';
            for ($i = 1; $i <= 10; $i++) {
            	$token .= substr($chars, rand(0, strlen($chars)-1), 1);
            }                
            return $token;
        }

        function getHolidays($day, $showHoliday = false) {

            if (in_array($day, $this->options['holidays'])) {
                if ($showHoliday) {
                    return 'data-holiday';
                } else {
                    return 'data-week';
                }
            }

            if (in_array($day, $this->options['weekends'])) {
                return 'data-week';
            }
            if (in_array($day, $this->options['days'])) {
                return '';
            }
            
            $class = array();
            $n = date('N', strtotime($day));
            if (($n == 6) or ($n == 7)) {
                $class[] = 'data-week';
                
                if ($this->options['first_day'] == 'sunday') {
                    $class[] = 'data-first';
                }
                return implode(' ', $class);
            }
            if (($n == 1) and ($this->options['first_day'] == 'monday')) {
                
                return 'data-first';
            }
            return '';
        }

        function prepareHeadLine() {
            $html = "\r\n". 
                    '<tr class="header-line">'. "\r\n" .
                    "\t" . '<th></th>' . "\r\n";
            if ((!isset($this->options['info'])) or ($this->options['info'] != 'hide')) {
                $html .= "\t" . '<th data-info data-right>' .  __('FD', 'shift-schedule') . '</th>' . "\r\n";
            }
            $now = $this->_start;
            while ($now <= $this->_end) {
                $day = date_i18n('d', strtotime($now));
                $this->month = date_i18n('Y-m', strtotime($now));

                $class = $this->getHolidays($this->month.'-'.$day, true);
                if ($this->check_works) {
                    if (($class == '') or ($class == 'data-first')) {
                        $this->work_day_in_month++;
                    }
                }
                
                $html .= "\t" . '<th '.$class.'>'. date_i18n('d', strtotime($now));
                if (isset($this->options['week'])) {
                    $html .= '<br /><span class="day-name">'.date_i18n('l', strtotime($now)).'</span>';
                }
                $html .= '</th>' . "\r\n";
                
                $now = date('Y-m-d', strtotime('+1 day', strtotime($now)));
            }

            if ((!isset($this->options['info'])) or ($this->options['info'] != 'hide')) {
                $html .= "\t" . '<th data-info>&Sigma;</th>' . "\r\n" .
                         "\t" . '<th data-info>+</th>' . "\r\n";
                foreach ((array) $this->options['types'] as $type) {
                    if ($type->sum_row == 1) {
                        $html .= "\t" . '<th data-info>'.$type->chars.'</th>' . "\r\n";
                    }
                }
            }
            $html .= '</tr>' . "\r\n";

            $this->check_works = false;

            return $html;
        }
        
        function prepareSumLine() {

            $in = date_create($this->_start);
            $out = date_create($this->_end);
            $diff = date_diff($in, $out);
            $days =  $diff->d + 3;
            $html = "\r\n". 
                    '<tr class="sum-line">'. "\r\n" .
                    "\t" . '<td colspan="'.$days.'" style="text-align:right;">'.__('Sum:', 'shift-schedule').'</td>' . "\r\n" .
                    "\t" . '<td></td>' . "\r\n" . 
                    "\t" . '<td class="sum_plus" data-info></td>' . "\r\n";
            foreach ((array) $this->options['types'] as $type) {
                if ($type->sum_row == 1) {
                    $html .= "\t" . '<td class="sum_'.$type->alias.'" data-info></td>' . "\r\n";
                }
            }
            $html .= '</tr>' . "\r\n";

            return $html;
        }


        function prepareSumRow() {
            $html = '';
            $sum_colls = 0;
            foreach ((array) $this->options['types'] as $type) {
                if ($type->sum_row == 1) {
                    $sum_colls++;
                }
            }

            foreach ((array) $this->options['types'] as $type) {
                if ($type->sum_row == 1) {
                    $html .= '<tr>'. "\r\n" . 
                             '<td class="name" colspan="2" data-right>'.$type->title.'</td>' . "\r\n";
                    $now = $this->_start;
                    while ($now <= $this->_end) {
                        $day = date_i18n('d', strtotime($now));
                        $this->month = date_i18n('Y-m', strtotime($now));
        
                        $class = $this->getHolidays($now);
                        $html .= "\t" . '<td class="sum_'.$type->alias.'_'.$day.'" '.$class.'></td>' . "\r\n";
                        
                        $now = date('Y-m-d', strtotime('+1 day', strtotime($now)));
                    }

/*
                    for ($i = 1; $i <= $this->_end; $i++) {
                    	$day = $i;
                        if ($i < 10) { $day = '0'.$i; }
                        $class = $this->getHolidays($this->month.'-'.$day);
                        $html .= "\t" . '<td class="sum_'.$type->alias.'_'.$day.'" '.$class.'></td>' . "\r\n";
                    }
*/
                    $html .= "\t". '<td colspan="'.($sum_colls+2).'"></td>' . "\r\n" .
                             '</tr>' . "\r\n";
                }
            }            
            return $html;
        }

        function prepareUsersLine() {
            
            global $wpdb;

            $users = array();
            $html = '';
            $table_name = $wpdb->prefix . 'shift_schedule';
            
            if (is_admin()) {

                switch ($this->category){ 
                	case 0:
                        $users = get_posts([
                            'post_type' => 'schedule-user',
                            'post_status' => 'publish',
                            'posts_per_page' => -1
                        ]);            
                    	break;
                
                	case '-1':
                        $users = get_posts([
                            'post_type' => 'schedule-user',
                            'post_status' => 'publish',
                            'posts_per_page' => -1,
                            'tax_query' => array( 
                                 array(
                                    'taxonomy' => 'schedule-category',
                                    'field' => 'term_id',
                                    'operator' => 'NOT IN',
                                    'terms' => get_terms('schedule-category', array(
                                        'fields' => 'ids'
                                        ))
                                    )
                                 )
                            ]);            
                    	break;
                
                
                	default :
                        $users = get_posts([
                            'post_type' => 'schedule-user',
                            'post_status' => 'publish',
                            'posts_per_page' => -1,
                            'tax_query' => array( 
                                 array(
                                    'taxonomy' => 'schedule-category',
                                    'terms' => $this->category,
                                    'include_children' => false)
                                )
                            ]);            
                    
                }            

            } else {

                if ((isset($this->options['category'])) and (!empty($this->options['category']))) {
                    
                    $cats = str_replace(' ', '', $this->options['category']);
                    $cats = explode(',', $cats);
                    $users = get_posts([
                        'post_type' => 'schedule-user',
                        'post_status' => 'publish',
                        'posts_per_page' => -1,
                        'tax_query' => array(
                            array(
                                'taxonomy' => 'schedule-category',
                                'field' => 'slug',
                                'terms' => $cats,
                                'operator' => 'IN'
                            ))
                    ]);            
                } else {
                    $users = get_posts([
                        'post_type' => 'schedule-user',
                        'post_status' => 'publish',
                        'posts_per_page' => -1,
                    ]);            
                }
            }

            if (empty($users)) {
                $sum_colls = 0;
                foreach ($this->options['types'] as $type) {
                    if ($type->sum_row == 1) {
                        $sum_colls++;
                    }
                }
                $html = '<tr>'. "\r\n" . 
                         '<td colspan="100" align="center">'.__('There are no users registered for this schedule.', 'shift-schedule').'</td>' . "\r\n" . 
                         '</tr>';
            } else {
                for ($i = 0; $i <= count($users)-1; $i++) {

                    $hour = get_post_meta( $users[$i]->ID, 'custom_hours', true );
                    if (empty($hour)) { $hour = 0; }
                    $free = get_post_meta( $users[$i]->ID, 'custom_free', true );
                    
                    $day = $this->_start;
                    if ($day < 10) { $day = '0'.$day; } 

                    $query = 'SELECT `day`, `flag`, `boss` FROM `'.$table_name.'` ' .
                             'WHERE ' .
                                '`user_id` = "'.$users[$i]->ID.'" AND ' .
                                '`day` >= "'.$this->_start.'" AND '.
                                '`day` <= "'.$this->_end.'"';
                    $data = $wpdb->get_results($query, ARRAY_A);
                    $days = array();
                    foreach ($data as $item) {
                        $days[$item['day']] = array('flag' => $item['flag'], 'boss' => $item['boss']);
                    }
                    if (has_filter('shift_schedule_render_name')) {
                        $name = apply_filters('shift_schedule_render_name', $users[$i]);
                    } else {
                        $name = $users[$i]->post_title;    
                    }
                    
                    $html .= '<tr data-id="'.$users[$i]->ID.'" data-hour="'.($this->work_day_in_month * $hour).'">' . "\r\n" .
                             "\t" . '<td class="name">'.$name.'</td>' . "\r\n";
                    if ((!isset($this->options['info'])) or ($this->options['info'] != 'hide')) {                             
                        $html .= "\t" . '<td id="user_free_'.$users[$i]->ID.'" data-right data-info>'.$free.'</td>' . "\r\n";
                    }

                    $now = $this->_start;
                    while ($now <= $this->_end) {
                        $day = date_i18n('d', strtotime($now));
                        $this->month = date_i18n('Y-m', strtotime($now));

                        $class = array('day');
                        if (isset($days[$now]['flag'])) {
                            $class[] = $days[$now]['flag'];
                        }
                        
                        $flag = $this->getHolidays($now);
                        if (isset($days[$now]['boss'])) {
                            if ($days[$now]['boss'] == '1') {
                                $flag .= ' data-boss';                                
                            }
                        }
                        
                        $html .= "\t" . '<td class="'.implode(' ', $class).'" '.$flag.' data-day="'.$day.'"></td>' . "\r\n";
                        
                        $now = date('Y-m-d', strtotime('+1 day', strtotime($now)));
                    }

                    if ((!isset($this->options['info'])) or ($this->options['info'] != 'hide')) {
                        $html .= "\t" . '<td class="user_'.$users[$i]->ID.'_sum" data-info></td>' . "\r\n" .
                                 "\t" . '<td class="line-plus user_'.$users[$i]->ID.'_plus" data-info></td>' . "\r\n";
                        foreach ((array) $this->options['types'] as $type) {
                            if ($type->sum_row == 1) {
                                $html .= "\t" . '<td class="user_'.$users[$i]->ID.'_'.$type->alias.' line-'.$type->alias.'" data-info></td>' . "\r\n";
                            }
                        }
                    }
                             
                    $html .= '</tr>' . "\r\n";                             
                                                        	
                }
            }

            return $html;
        }

        function prepareLegend() {
            $html = array();
            $colls = 0;
            
            foreach ((array) $this->options['types'] as $type) {
                if ($type->sum_row == 1) {
                    $colls++;
                }
            }

            foreach ((array) $this->options['types'] as $type) {
                $html[] = '<div class="flag" style="background-color:'.$type->color.';">'.$type->chars.'</div>' .
                            $type->title . "\r\n";
            }

            $in = date_create($this->_start);
            $out = date_create($this->_end);
            $diff = date_diff($in, $out);
            $days =  $diff->d + 5;

            $html = "\t" . '<tr class="legend">' . "\r\n" .
                    "\t\t" . '<td colspan="'.($colls + $days).'">' . implode('&nbsp;&nbsp;&nbsp;&nbsp;', $html) . '</td>' . "\r\n" .
                    "\t" . '</tr>' . "\r\n";
            return $html;
        }

        function prepareContent()
        {   
            $html = '<form id="shift-schedule-form">';
            $html .= '<table style="width: 100%;">' . "\r\n";
            $html .= '    <tr>' . "\r\n";
            if (!$this->silent) {
                $html .= '  <td style="width: 30%; text-align: left;">' . "\r\n";
                $html .= '        <button onclick="shift_schedule_698_setCookie(\''.date( 'Y-m', strtotime('-1 month', strtotime($this->month.'-01'))).'\');jQuery(\'#shift_schedule_month\').val(\''.date( 'Y-m', strtotime('-1 month', strtotime($this->month.'-01'))).'\');" class="button" >' . "\r\n";
                $html .= '            <span class="dashicons dashicons-arrow-left" style="padding-top:5px"></span>' . "\r\n";
                $html .= '            '.date_i18n( 'F', strtotime('-1 month', strtotime($this->month.'-01'))) . "\r\n";
                $html .= '        </button>' . "\r\n";
                $html .= '    </td>' . "\r\n";        
            }            
            $html .= '        <td style="text-align: center;">' . "\r\n";
            $html .= '            <h3 id="schedule_title">'.date_i18n( 'Y. F', strtotime($this->month.'-01')).'</h3>' . "\r\n";
            $html .= '        </td>' . "\r\n";
            if (!$this->silent) {
                $html .= '  <td style="width: 30%; text-align: right;">' . "\r\n";
                $html .= '        <button onclick="shift_schedule_698_setCookie(\''.date( 'Y-m', strtotime('+1 month', strtotime($this->month.'-01'))).'\');jQuery(\'#shift_schedule_month\').val(\''.date( 'Y-m', strtotime('+1 month', strtotime($this->month.'-01'))).'\');" class="button" >' . "\r\n";
                $html .= '            '.date_i18n( 'F', strtotime('+1 month', strtotime($this->month.'-01'))) . "\r\n";
                $html .= '            <span class="dashicons dashicons-arrow-right" style="padding-top:5px"></span>' . "\r\n";
                $html .= '        </button>' . "\r\n";
                $html .= '    </td>' . "\r\n";        
            }            
            $html .= '    </tr>' . "\r\n";       
            if (!$this->silent) {
                $html .= '<tr>' . "\r\n";
                $html .= '    <td colspan="2">' . "\r\n";
                $html .= __('Category:', 'shift-schedule') . '&nbsp;';
                
                $selected = 0;
                if (isset($_REQUEST['shift-schedule-category-selector'])) {
                    $selected = sanitize_text_field($_REQUEST['shift-schedule-category-selector']);
                }
                $html .= wp_dropdown_categories(
                        array(
                            'name' => 'shift-schedule-category-selector',
                            'id' => 'shift_schedule_category_selector',
                            'option_none_value' => '-1',
                            'show_option_all' => __('Show all', 'shift-schedule'),
                            'show_option_none' => __('Not set category', 'shift-schedule'),
                            'taxonomy' => 'schedule-category',
                            'hierarchical' => 1,
                            'echo' => 0,
                            'selected' => $selected
                        )
                    );
                $month = '';
                if (isset($_REQUEST['month'])) {
                    $month = sanitize_text_field($_REQUEST['month']);
                }

                $html .= '<input type="hidden" name="page" value="shift-schedule-setting-admin" />';
                $html .= '<input type="hidden" name="shift-schedule-tab" value="schedule" />';
                $html .= '<input type="hidden" id="shift_schedule_month" name="month" value="'.$month.'" />';
                $html .= '    </td>' . "\r\n";
                $html .= '    <td style="text-align: right;padding-bottom:12px;">' . "\r\n";

                if (!empty($this->options['types'])) {

                    $html .= __('Select shift type:', 'shift-schedule');

                    $checks = array('<select id="day_type" name="day_type">');
                    $day_type = '';
                    if (isset($_REQUEST['day_type'])) {
                        $day_type = sanitize_text_field($_REQUEST['day_type']);
                    }
                    foreach ((array) $this->options['types'] as $type) {
                        if ($day_type == $type->alias) {
                            $selected = 'selected="selected"';
                        } else {
                            $selected = '';
                        }
                        $checks[] = "\t" . '<option value="'.$type->alias.'" '.$selected.'>'.$type->title.'</option>';
                    }
                    $checks[] = '</select>';
                    $html .= "\r\n" . implode("\r\n", $checks) . "\r\n";                        
                    $html .= '<br /><small>'. __('Select a type and then click in the table to set it.', 'shift-schedule').'</small>' . "\r\n";
                } else {
                    $html .=  '<span style="color:red">' . __('No locations are available.', 'shift-schedule') . '</span>' . "\r\n" . 
                             '<br /><a href="'.get_admin_url().'options-general.php?page=shift-schedule-setting-admin&shift-schedule-tab=settings">' . 
                              __('Define these first!', 'shift-schedule') . '</a>' . "\r\n";
                }
                
                $html .= '    </td>' . "\r\n";
                $html .= '</tr>' . "\r\n";   
            }      
            $html .= '</table>' . "\r\n";
            $html .= '</form>';
            
            $html .= '<table id="schedules_'.$this->options['token'].'" cellspacing="1" class="schedules '.$this->display_mode.'" data-token="'.$this->options['token'].'" data-month="'.$this->month.'">' . "\r\n";
            $html .= '    <thead>' . "\r\n";
            $html .= $this->prepareHeadLine();
            $html .= '    </thead>' . "\r\n";
            $html .= '    <tbody>' . "\r\n";
            $html .= $this->prepareUsersLine(); 
            $html .= '    </tbody>' . "\r\n";
            $html .= '<tfoot>' . "\r\n";
            if (!$this->silent) {
                $html .= $this->prepareSumLine();
                $html .= $this->prepareHeadLine();
                $html .= $this->prepareSumRow(); 
            }
            $html .= $this->prepareLegend();
            $html .= '</tfoot>' . "\r\n";
            $html .= '</table>' . "\r\n";
            
            if (!$this->silent) {
                $html .= '<div>' . "\r\n";
                $html .= '    <small>'. __('Double-click to nominate a shift manager!', 'shift-schedule').'</small>' . "\r\n";
                $html .= '</div>' . "\r\n";

                $html .= '<div style="text-align: right;">' . "\r\n";
                $html .= '    <button onclick="shift_schedule_698_print();" class="button button-primary">' . "\r\n";
                $html .= '        <span id="shift_schedule_698_print_preview_button" class="dashicons dashicons-visibility" style="margin-top: 4px;"></span>' . "\r\n";
                $html .= '        '. __('Print preview', 'shift-schedule') . "\r\n";
                $html .= '    </button>' . "\r\n";
                $html .= '    <button onclick="shift_schedule_698_csv();" class="button button-primary">' . "\r\n";
                $html .= '        <span id="shift_schedule_698_csv_button" class="dashicons dashicons-editor-table" style="margin-top: 4px;"></span>' . "\r\n";
                $html .= '        '. __('Export CSV', 'shift-schedule') . "\r\n";
                $html .= '    </button>' . "\r\n";
                $html .= '</div>' . "\r\n";
                
            }

            return $html;

        }
        
    }