<?php

if ( !defined( 'ABSPATH' ) ) exit;

class shift_schedule_698_settings
{
    private $options;

    public function __construct()
    {
        add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
        add_action( 'admin_init', array( $this, 'page_init' ) );
    }

    public function add_plugin_page()
    {
        add_options_page(
            'Settings Admin', 
            __('Shift schedule settings', 'shift-schedule'), 
            'manage_options', 
            'shift-schedule-setting-admin', 
            array( $this, 'create_admin_page' )
        );
    }

    function getAlias($text)
    {
        $str = trim(mb_strtolower($text, 'UTF-8'));

        $accented = array( "á", "é", "í", "ó", "ö", "ő", "ú", "ü", "ű" );  
        $mask     = array( "a", "e", "i", "o", "o", "o", "u", "u", "u" );  
        $str = str_replace( $accented, $mask, $str );  

        $str = str_replace('-', ' ', $str);
        $str = str_replace('.', ' ', $str);
        $str = str_replace('_', ' ', $str);
        $str = preg_replace(array('/\s+/','/[^A-Za-z0-9\-]/'), array('-',''), $str);
        
        return $str;
    }

    public function create_admin_page()
    {
        $this->options = shift_schedule_698_get_options ( 'shift_schedule_options' );
        ?>
        <style>.form-table td {padding: 0px 10px 0 10px;}</style>
        <div class="wrap">
            <h1><?php echo __('Shift schedule settings', 'shift-schedule');?></h1>
            <?php
                $active_tab = isset( $_GET[ 'shift-schedule-tab' ] ) ? sanitize_text_field($_GET[ 'shift-schedule-tab' ]) : 'schedule';
            ?>
            <h2 class="nav-tab-wrapper">
                <a href="<?php echo get_admin_url();?>options-general.php?page=shift-schedule-setting-admin&shift-schedule-tab=schedule" class="nav-tab  <?php echo $active_tab == 'schedule' ? 'nav-tab-active' : ''; ?>"><?php echo __('Schedule month', 'shift-schedule');?></a>
                <a href="<?php echo get_admin_url();?>edit.php?post_type=schedule-user" class="nav-tab  <?php echo $active_tab == 'users' ? 'nav-tab-active' : ''; ?>"><?php echo __('Schedule users', 'shift-schedule');?></a>
                <a href="<?php echo get_admin_url();?>options-general.php?page=shift-schedule-setting-admin&shift-schedule-tab=settings" class="nav-tab  <?php echo $active_tab == 'settings' ? 'nav-tab-active' : ''; ?>"><?php echo __('Plugin settings', 'shift-schedule');?></a>
            </h2>
            <div class="shift-schedule-setting-page">
                <div>
            <?php
                switch ($active_tab){ 

                    case 'settings':
                        ?>
                        <form method="post" action="options.php">
                        <?php
                            settings_fields( 'shift_schedule_option_main' );
                            do_settings_sections( 'shift-schedule-setting-admin' );
                            submit_button();
                        ?>
                        </form>
                        <?php
                        exit;

                    default: 
                        if (!class_exists('JShiftSchedule698')) {
                            include( dirname(__FILE__) . DIRECTORY_SEPARATOR . 'class.php' );
                        }
                        $schedule = new JShiftSchedule698($this->options);
                        $html = $schedule->prepareContent();
                        echo $html;
                }
            ?>
                </div>
            </div>
        </div>
        <?php
    }

    public function page_init()
    {        
        register_setting(
            'shift_schedule_option_main', 
            'shift_schedule_options', 
            array( $this, 'sanitize' ) 
        );

        add_settings_section(
            'shift_schedule_settings_main', 
            __('Shift schedule Custom Settings', 'shift-schedule'), 
            array( $this, 'print_section_info' ), 
            'shift-schedule-setting-admin'
        );  

        add_settings_field(
            'first_day', 
            __('First day in week', 'shift-schedule'),
            array( $this, 'first_day_callback' ), 
            'shift-schedule-setting-admin', 
            'shift_schedule_settings_main' 
        );

        add_settings_field(
            'holidays', 
            __('Holidays', 'shift-schedule'), 
            array( $this, 'holidays_callback' ),
            'shift-schedule-setting-admin',
            'shift_schedule_settings_main' 
        );

        add_settings_field(
            'weekends', 
            __('Weekends', 'shift-schedule'), 
            array( $this, 'weekends_callback' ),
            'shift-schedule-setting-admin',
            'shift_schedule_settings_main' 
        );

        add_settings_field(
            'days', 
            __('Working days', 'shift-schedule'), 
            array( $this, 'days_callback' ), 
            'shift-schedule-setting-admin', 
            'shift_schedule_settings_main'
        );

        add_settings_field(
            'types',
            __('Types', 'shift-schedule'), 
            array( $this, 'types_callback' ), 
            'shift-schedule-setting-admin', 
            'shift_schedule_settings_main'
        );


    }

    public function sanitize( $input )
    {
        $new_input = array();

        if ( isset( $input['first_day'] ) ) {
            $new_input['first_day'] = trim( $input['first_day'] );
        }
        if ( isset( $input['holidays'] ) ) {
            $holidays = str_replace(' ', '', $input['holidays']);
            $holidays = explode(',', $holidays);
            $new_input['holidays'] = $holidays;
        }
        if ( isset( $input['weekends'] ) ) {
            $weekends = str_replace(' ', '', $input['weekends']);
            $weekends = explode(',', $weekends);
            $new_input['weekends'] = $weekends;
        }
        if ( isset( $input['days'] ) ) {
            $days = str_replace(' ', '', $input['days']);
            $days = explode(',', $days);
            $new_input['days'] = $days;
        }

        $types = array();
        if ( isset( $input['types'] ) ) {
            $types = json_decode($input['types']);
            
            if (!empty($types)) {
                for ($i = 0; $i <= count($types)-1; $i++) {
                	if (empty($types[$i]->alias)) {
                	   $types[$i]->alias = $this->getAlias($types[$i]->title);
                	}
                }
            }
        }
        $new_input['types'] = $types;


        return $new_input;
    }

    public function print_section_info()
    {
        print __('These settings determine the framework in which the plugin works.', 'shift-schedule');
    }

    public function first_day_callback()
    {
        echo '<!-- ' . __('Monday', 'shift-schedule') . ' -->';
        echo '<!-- ' . __('Sunday', 'shift-schedule') . ' -->';
        $values = array('monday', 'sunday');
        $html = array();
        foreach ($values as $item) {
            $tmp = '<input type="radio" name="shift_schedule_options[first_day]" value="'.$item.'" ';
            if ($this->options['first_day'] == $item) {
                $tmp .= 'checked="checked" ';
            }
            $tmp .= '/>' . __(ucfirst(strtolower($item)), 'shift-schedule');
            $html[] = $tmp;
        }
        $html = '<div>' . implode('<br />', $html) . '</div>';
        $html .= '<small>'.__('The first day of the week', 'shift-schedule').'</small>';
        $html .= '<div style="clear:both;margin-bottom:10px;"></div>';
        echo $html;
    }

    public function holidays_callback()
    {
        $holidays = '';
        if (isset( $this->options['holidays'])) {
            $holidays = implode(', ', $this->options['holidays']);
        }
        printf(
            '<textarea name="shift_schedule_options[holidays]">%s</textarea>' .
            '<br /><small>'.__('Comma-separated list of holidays. <em>(Ex: 2020-08-20, 2020-10-23)</em>', 'shift-schedule').'</small>',
            $holidays
        );
        echo '<div style="clear:both;margin-bottom:10px;"></div>';

    }

    public function weekends_callback()
    {
        $weekends = '';
        if (isset( $this->options['weekends'])) {
            $weekends = implode(', ', $this->options['weekends']);
        }
        printf(
            '<textarea name="shift_schedule_options[weekends]">%s</textarea>' .
            '<br /><small>'.__('Days that will be days off due to festive work schedules.', 'shift-schedule').'</small>',
            $weekends
        );
        echo '<br /><small>'.__('List of weekends separated by commas. (Ex: 2020-08-21, 2020-12-24)', 'shift-schedule').'</small>';
        echo '<div style="clear:both;margin-bottom:10px;"></div>';
    }

    public function days_callback()
    {
        $days = '';
        if (isset( $this->options['days'])) {
            $days = implode(', ', $this->options['days']);
        }
        printf(
            '<textarea name="shift_schedule_options[days]">%s</textarea>' .
            '<br /><small>'.__('Days that will be working days due to a holiday schedule.', 'shift-schedule').'</small>',
            $days
        );
        echo '<br /><small>'.__('List of working days separated by commas. (Ex: 2020-08-29, 2020-12-12)', 'shift-schedule').'</small>';
        echo '<div style="clear:both;margin-bottom:10px;"></div>';
    }

    public function types_callback()
    {
        $types = $this->options['types'];
        echo __('These types will be selectable for one person per day.', 'shift-schedule');        
        ?>
        <ul class="sortable list-unstyled" id="shift_schedule_698_types">
        <?php
        if (empty($types)) {
            echo '<div id="no_types"><small>' . __('No types configured.', 'shift-schedule') . '</small></div>';
        } else {
            for ($i = 0; $i <= count($types)-1; $i++) {

                echo '<li class="admin-types" ' .
                            'id="types_'.$i.'" ' .
                            'data-title="'.$types[$i]->title.'" ' .
                            'data-alias="'.$types[$i]->alias.'" ' .
                            'data-hours="'.$types[$i]->hours.'" ' .
                            'data-color="'.$types[$i]->color.'" ' .
                            'data-chars="'.$types[$i]->chars.'" ' .
                            'data-free="'.$types[$i]->free.'" ' .
                            'data-sum-row="'.$types[$i]->sum_row.'" ' .
                            'data-sum-col="'.$types[$i]->sum_col.'" ' .
                            '>' .
                       '<div class="delete-icon" onclick="shift_schedule_698_deleteType(jQuery(this))"><span class="small dashicons dashicons-trash"></span></div>' .
                       '<div class="edit-icon" onclick="shift_schedule_698_editTypeBox(jQuery(this))"><span class="small dashicons dashicons-edit"></span></div>' .
                       '<div class="type-info">' .
                       '<div class="type-color-demo" style="background-color:'.$types[$i]->color.'"></div>' .
                       $types[$i]->title . ' (' . $types[$i]->chars . ') <code><small>[' . $types[$i]->alias . ']</small></code>' . 
                       '</div>' . 
                       '</div>' .
                   '</li>';
            }
        }
         
        
        ?>
        </ul>
        <div style="clear: both;"></div>
        
        <div style="text-align: right; margin-top: 6px;">
            <button onclick="shift_schedule_698_showNewTypeBox()" type="button" class="button button-secondary">
                <span id="shift_schedule_698_add_new_type" class="dashicons dashicons-plus" style="margin-top:5px;"></span>
                <?php echo __('Add new type', 'shift-schedule');?>           
            </button>
        </div>

        <div id="get_types_properties" style="display:none;">
            <form id="title_box" action="#"> 
                <div>
                    <table>
                        <tr>
                            <td align="right">
                                <label class="types_title_label" for="types_title"><?php echo __('Schedule title', 'shift-schedule');?>:&nbsp;</label>
                            </td>
                            <td align="left">
                                <input id="types_id" type="hidden" value="" style="width:180px;" />
                                <input id="types_title" type="text" value="" style="width:180px;" />
                                <input id="types_alias" type="hidden" value="" style="width:180px;" />
                            </td>
                        </tr>
                        <tr>
                            <td align="right" valign="top">
                                <label class="types_hours_label" for="types_hours"><?php echo __('Work Hours','shift-schedule');?>:&nbsp;</label>
                            </td>
                            <td align="left">
                                <input id="types_hours" type="text" value=""  style="width:60px;" />
                                <div><small><?php echo __('You count that many hours on this position.', 'shift-schedule');?></small></div>
                            </td>
                        </tr>
                        <tr>
                            <td align="right" valign="top">
                                <label class="types_hours_label" for="types_color"><?php echo __('View','shift-schedule');?>:&nbsp;</label>
                            </td>
                            <td align="left">
                                <?php echo __('Color','shift-schedule');?>:&nbsp;<input id="types_color" type="color" value="#ffffff" />&nbsp;&nbsp;&nbsp;
                                <?php echo __('Character','shift-schedule');?>:&nbsp;<input id="types_chars" type="text" value="" style="width:40px;" />
                            </td>
                        </tr>
                        <tr>
                            <td colspan="2">
                                <hr />
                            </td>
                        </tr>
                        <tr>
                            <td align="right" valign="top">
                                <label class="types_hours_label" for="types_is_free"><?php echo __('Free day','shift-schedule');?>:&nbsp;</label>
                            </td>
                            <td align="left">
                                <input id="types_is_free" type="checkbox" value="1" />&nbsp;<?php echo __('Free day schedule','shift-schedule');?>
                                <div><small><?php echo __('Changes the number of days off.', 'shift-schedule');?></small></div>
                            </td>
                        </tr>
                        <tr>
                            <td align="right" valign="top">
                                <label class="types_hours_label" for="types_sum_row"><?php echo __('Sum by row','shift-schedule');?>:&nbsp;</label>
                            </td>
                            <td align="left">
                                <input id="types_sum_row" type="checkbox" value="1" />&nbsp;<?php echo __('Aggregation by row.','shift-schedule');?>
                            </td>
                        </tr>
                        <tr>
                            <td align="right" valign="top">
                                <label class="types_hours_label" for="types_sum_col"><?php echo __('Sum by day','shift-schedule');?>:&nbsp;</label>
                            </td>
                            <td align="left">
                                <input id="types_sum_col" type="checkbox" value="1" />&nbsp;<?php echo __('Aggregation by day.','shift-schedule');?>
                            </td>
                        </tr>
                    </table>
                    <div class="message_functions" style="text-align:right">
                        <input id="add_new_type_button" onclick="shift_schedule_698_addNewType();" type="submit" class="button button-primary" value="<?php echo __('Add type', 'shift-schedule');?>" />                       
                        <input id="edit_type_button" onclick="shift_schedule_698_editType();" type="submit" class="button button-primary" value="<?php echo __('Edit type', 'shift-schedule');?>" />                       
                        <input onclick="shift_schedule_698_hideTypesBox();" type="button" class="button button-secondary" value="<?php echo __('Cancel');?>" />
                    </div>
                </div>
            </form> 
        </div>
        
        <input type="hidden" name="shift_schedule_options[types]" id="types" value="" />
        <?php


    }

}
