<?php
/*
Plugin Name: Shift schedule
Plugin URI:  http://wpdemo.dovi42.hu/shift-schedule
Description: Create shift shifts easily, in one locations.
Version:     1.38
Author:      Dovi42
Author URI:  http://dovi42.hu/
License:     GPLv2 or later
License URI: https://www.gnu.org/licenses/gpl-2.0.html
Text Domain: shift-schedule
Generator:   WP Creator - http://wpcreator.hu - 2020-07-24 06:56:26
*/

if ( !defined( 'ABSPATH' ) ) exit;

if (!defined('DS')) { define( 'DS', DIRECTORY_SEPARATOR ); }

define('SHIFT_SCHEDULE_698_ROOT', dirname(__FILE__));
define('SHIFT_SCHEDULE_698_VERSION', '1.38');

function shift_schedule_698_get_options() { 

	$defaults = array(
			'first_day' => 'monday',
			'holidays'  => array(),
			'weekends'  => array(),
			'days'      => array(),
			'types'     => array()
		);

	$options = get_option('shift_schedule_options');
	$options = wp_parse_args($options, $defaults);

	return $options;
}


function shift_schedule_698_load_textdomain() {
    load_plugin_textdomain( 'shift-schedule', false, basename( dirname( __file__ )) . '/languages' ); 
}
add_action( 'plugins_loaded', 'shift_schedule_698_load_textdomain' );

include_once( dirname(__file__) .DS. 'libs' .DS. 'define.php');

function shift_schedule_698_activation() {

	global $wpdb;
	$charset_collate = $wpdb->get_charset_collate();
    $table_name = $wpdb->prefix . 'shift_schedule';

	$sql = 'CREATE TABLE IF NOT EXISTS '.$table_name.' (
        		`id` int(11) NOT NULL AUTO_INCREMENT,
        		`user_id` int(11) NOT NULL,
                `day` varchar(10) NOT NULL DEFAULT "",
                `flag` varchar(15) NOT NULL DEFAULT "",
                `boss` tinyint(1) NOT NULL DEFAULT 0,
        		UNIQUE KEY id (id)
        	) ' . $charset_collate;
	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	dbDelta( $sql );

	$options = shift_schedule_698_get_options();
	update_option('shift_schedule_options', $options, true);
}
register_activation_hook( __file__, 'shift_schedule_698_activation' );

function shift_schedule_698_uninstall() {
    global $wpdb;
    $wpdb->query('DROP TABLE IF EXISTS ' . $wpdb->prefix . 'shift_schedule');
    $all = get_posts( 
            array(
                'post_type' => 'schedule-user',
                'numberposts' => -1) 
            );
    foreach ($all as $item) {
        wp_delete_post( $item->ID, true );
    }            
	delete_option('shift_schedule_options');

}
register_uninstall_hook( __file__, 'shift_schedule_698_uninstall' );



function shift_schedule_698_shortcode($args) {

	$options = shift_schedule_698_get_options();
    $args = wp_parse_args($args, $options);

    if (!isset($args['month'])) {
        $args['month'] = date_i18n('Y-m');
    }

    if ((isset($args['month'])) and (strtolower($args['month']) == 'last')) {
        global $wpdb;
        $query = 'SELECT `day` FROM `' . $wpdb->prefix . 'shift_schedule` ORDER BY `day` DESC LIMIT 1';
        $month = $wpdb->get_results($query, ARRAY_A);
        $args['month'] = substr($month[0]['day'], 0, 7);
    }

    if ((isset($args['month'])) and (strtolower($args['month']) == 'next')) {
        $args['month'] = date_i18n('Y-m', strtotime('+1 month'));
    }

	if (!defined('SHIFT_SCHEDULE_698')) {
		shift_schedule_698_styles_and_scripts_to_frontend();
	}

    if (!class_exists('JShiftSchedule698')) {
        include_once( dirname(__file__) .DIRECTORY_SEPARATOR. 'libs' .DIRECTORY_SEPARATOR. 'class.php');
    }
    $my_plugin = new JShiftSchedule698($args, true);

    $html = $my_plugin->prepareContent();
    
    if (WP_DEBUG) {
        $html .= '<!-- Shift schedule according to local time: ' . date_i18n('Y-m-d H:i:s') . ' -->';
    }
    
    return $html;
}
add_shortcode('shift-schedule', 'shift_schedule_698_shortcode');

function shift_schedule_698_check_date($x) {
    return (date('Y-m-d', strtotime($x)) == $x);
}

function shift_schedule_698_users_shortcode($args) {

    $day = date_i18n('Y-m-d');
    
    if (isset($args['day'])) {
        
        if (shift_schedule_698_check_date($args['day'])) {
            $day = $args['day'];
        } else {
            if ($args['day'] == 'tommorow') {
                $day = date_i18n('Y-m-d', strtotime('+1 day', strtotime(date_i18n('Y-m-d'))));
            } else {
                $day = date_i18n('Y-m-d', strtotime($args['day'].' day', strtotime(date_i18n('Y-m-d'))));
            }
        }
    }

    $flag = null;
    if (isset($args['filter'])) {
        $flag = $args['filter'];
    }

    $category = null;
    if (isset($args['category'])) {
        $category = $args['category'];
    }

    $users = shift_schedule_698_get_users($day, $flag, $category);

    $html = array();
    
    if (!empty($users)) {
        foreach ($users as $user) {
            if (has_filter('shift_schedule_show_user')) {
                $html[] = apply_filters('shift_schedule_show_user', $user['user_id'], $user['flag']); 
            }
        }
    } else {
        if (has_filter('shift_schedule_no_sunordinates')) {
            $html[] = apply_filters('shift_schedule_no_sunordinates', __('There are no subordinates', 'shift-schedule'));
        }
    }

    $html = implode('', $html);

    if (has_filter('shift_schedule_render_group')) {
        $html = apply_filters('shift_schedule_render_group', $html);    
    }

    if (has_filter('shift_schedule_render_properties')) {
        $html = apply_filters('shift_schedule_render_properties', $html, $day, $flag);    
    }

    if (WP_DEBUG) {
        $html .= '<!-- Shift schedule according to local time: ' . date_i18n('Y-m-d H:i:s') . ' -->';
    }

    return $html;
}
add_shortcode('shift-schedule-users', 'shift_schedule_698_users_shortcode');

function shift_schedule_698_styles_and_scripts_to_frontend()
{
    global $pagenow;

    define('SHIFT_SCHEDULE_698', 1);

    if (in_array( $pagenow, array( 'post.php', 'page.php', 'index.php' ))) {

        wp_register_style( 'shift-schedule', plugins_url( 'shift-schedule/assets/css/shift-schedule.css', SHIFT_SCHEDULE_698_ROOT ), array(), SHIFT_SCHEDULE_698_VERSION );
    	wp_enqueue_style( 'shift-schedule' );
        shift_schedule_698_inline_styles('shift-schedule');
        
        
    	wp_register_script('shift-schedule-js', plugins_url( 'shift-schedule/assets/js/shift-schedule.js', SHIFT_SCHEDULE_698_ROOT ), array('jquery'), SHIFT_SCHEDULE_698_VERSION, true);
        $options = shift_schedule_698_get_options();
        wp_localize_script('shift-schedule-js', 'shift_schedule_698_types', $options['types']);

		wp_localize_script('shift-schedule-js', 'shift_schedule_698_ajax_object', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );
		wp_enqueue_script('shift-schedule-js');
    }
}

if ( is_admin() ){ 

    global $pagenow, $post;
    
    include( dirname(__file__) .DIRECTORY_SEPARATOR. 'libs' .DIRECTORY_SEPARATOR. 'shift-schedule-setup.php' );
    $my_settings_page = new shift_schedule_698_settings();
    
    function shift_schedule_698_get_extra_meta_links($meta, $file, $data, $status) {

        if (plugin_basename(__FILE__) == $file) {
            $lang = get_bloginfo('language');
            if ($lang == 'hu') {
                $url = 'http://wpdemo.dovi42.hu/shift-schedule';
            } else {
                $url = 'http://wpdemo.dovi42.hu/shift-schedule-en';
            } 
            $meta[] = '<a href="'.$url.'" target="_blank" title="' . __('Detailed description and examples', 'shift-schedule') . '">'.__('Detailed description and examples', 'shift-schedule').'</a>';

            $meta[]  = '<a href="https://wordpress.org/support/plugin/shift-schedule/reviews/">'.
                       __('Please rating!', 'shift-schedule') . '&nbsp;' .
                       '<span class="dashicons dashicons-star-empty dashicons-star-filled" style="color:#ffb900 !important;"></span>'.
                       '<span class="dashicons dashicons-star-empty dashicons-star-filled" style="color:#ffb900 !important;"></span>'.
                       '<span class="dashicons dashicons-star-empty dashicons-star-filled" style="color:#ffb900 !important;"></span>'.
                       '<span class="dashicons dashicons-star-empty dashicons-star-filled" style="color:#ffb900 !important;"></span>'.
                       '<span class="dashicons dashicons-star-empty dashicons-star-filled" style="color:#ffb900 !important;"></span>'.
                       '</a>';
        }
        return $meta;
    }
    add_filter("plugin_row_meta", 'shift_schedule_698_get_extra_meta_links', 10, 4);

    $type = '';
    if (isset($_REQUEST['post_type']) and (!empty($_REQUEST['post_type']))) {
        $type = sanitize_text_field($_REQUEST['post_type']);
    }
    if ((in_array($pagenow, array('post.php'))) or ($type == 'schedule-user')) {
        include_once( dirname(__file__) .DS. 'libs' .DS. 'customize.php');
    }

    function shift_schedule_698_register_schedule_menu() {
        add_submenu_page(
            'edit.php?post_type=schedule-user', 
            __('Month schedule', 'shift-schedule'), 
            __('Month schedule', 'shift-schedule'), 
            'manage_options', 
            'options-general.php?page=shift-schedule-setting-admin'
        );
    }
    add_action('admin_menu', 'shift_schedule_698_register_schedule_menu');    
}

function shift_schedule_698_add_settings_link( $links ) {
    $settings_link = '<a href="options-general.php?page=shift-schedule-setting-admin">' . __( 'Settings' ) . '</a>';
    array_unshift( $links, $settings_link );
  	return $links;
}
$plugin = plugin_basename( __file__ );
add_filter( "plugin_action_links_$plugin", 'shift_schedule_698_add_settings_link' );

function shift_schedule_698_styles_and_scripts_to_backend()
{
    global $pagenow;
    
    if (isset($_REQUEST['page'])) {
        $page = sanitize_text_field($_REQUEST['page']);
    } else {
        $page = '';
    }
    if (($pagenow == 'options-general.php') and ($page == 'shift-schedule-setting-admin')) {

        $options = shift_schedule_698_get_options();
        
        wp_register_style( 'shift-schedule-admin', plugins_url( 'shift-schedule/assets/css/shift-schedule-admin.css', SHIFT_SCHEDULE_698_ROOT ), array(), SHIFT_SCHEDULE_698_VERSION);
    	wp_enqueue_style( 'shift-schedule-admin' );

        wp_register_style( 'shift-schedule', plugins_url( 'shift-schedule/assets/css/shift-schedule.css', SHIFT_SCHEDULE_698_ROOT ), array(), SHIFT_SCHEDULE_698_VERSION );
    	wp_enqueue_style( 'shift-schedule' );
        
    	wp_enqueue_script('jquery-ui-sortable', '', array('jquery'));
        wp_register_script('shift-schedule-admin-js', plugins_url( 'shift-schedule/assets/js/shift-schedule-admin.js', SHIFT_SCHEDULE_698_ROOT ), array('jquery-ui-sortable'), SHIFT_SCHEDULE_698_VERSION);

		wp_localize_script('shift-schedule-admin-js', 'shift_schedule_698_ajax_object', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );
        wp_localize_script('shift-schedule-admin-js', 'shift_schedule_698_types', $options['types']);
		wp_enqueue_script('shift-schedule-admin-js');

        wp_register_script('shift-schedule-js', plugins_url( 'shift-schedule/assets/js/shift-schedule.js', SHIFT_SCHEDULE_698_ROOT ), array('jquery-ui-sortable'), SHIFT_SCHEDULE_698_VERSION);
        wp_enqueue_script('shift-schedule-js');
        
        shift_schedule_698_inline_styles('shift-schedule-admin');
    }
}
add_action( 'admin_enqueue_scripts', 'shift_schedule_698_styles_and_scripts_to_backend' );

function shift_schedule_698_inline_styles($handle) {
    $options = shift_schedule_698_get_options();
    
    if (!empty($options['types'])) {
        $custom_css = array();
        foreach ($options['types'] as $type) {
            $custom_css[] = 'table.schedules td.'.$type->alias.' { background-color: ' .$type->color. ' !important; }';
            $custom_css[] = 'table.schedules td.'.$type->alias.':after { content: "' .$type->chars. '"; }';
        }
        $custom_css = implode("\r\n", $custom_css);
        wp_add_inline_style( $handle, $custom_css );        
    }
}

include(SHIFT_SCHEDULE_698_ROOT . '/libs/libs.php');
include(SHIFT_SCHEDULE_698_ROOT . '/libs/ajax.php');
include(SHIFT_SCHEDULE_698_ROOT . '/libs/hooks.php');